/* $Id$ */
#include <fstream>
#include <iostream>
#include <msp/core/application.h>
#include <msp/core/getopt.h>
#include "source/parser.h"
#include "source/statement.h"
#include "source/writer.h"

using namespace std;
using namespace Msp;

class DataTool: public Application
{
private:
	string in_fn;
	string out_fn;
	bool binary;
public:
	DataTool(int argc, char **argv);
	int main();

	static Application::RegApp<DataTool> reg;
};


DataTool::DataTool(int argc, char **argv):
	out_fn("-")
{
	GetOpt getopt;
	getopt.add_option('o', "output", out_fn, GetOpt::REQUIRED_ARG);
	getopt.add_option('b', "binary", binary, GetOpt::NO_ARG);
	getopt(argc, argv);

	const vector<string> &args=getopt.get_args();
	if(args.empty())
		throw UsageError("Must give input filename");

	in_fn=args[0];
}

int DataTool::main()
{
	ifstream in(in_fn.c_str());
	if(!in)
	{
		cerr<<"Couldn't open input file\n";
		return 1;
	}

	ostream *out;
	if(out_fn=="-")
		out=&cout;
	else
		out=new ofstream(out_fn.c_str());

	DataFile::Parser parser(in, in_fn);
	DataFile::Writer writer(*out);
	if(binary)
		writer.set_binary(true);

	while(parser)
	{
		DataFile::Statement st=parser.parse();
		if(st.valid)
			writer.write(st);
	}

	if(out!=&cout)
		delete out;

	return 0;
}

Application::RegApp<DataTool> DataTool::reg;
